/* test-file.c - unit tests for file.c
 *
 * Copyright (C) 2013  Jochen Voss, Andreas Voss.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 */

#undef NDEBUG

#include "file.c"

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <assert.h>

#define TEST_FILE_NAME "file-test-data"

int
main()
{
	FILE *fp;
	const char *const *tokens;
	int n_tokens, rc;

	fp = fopen(TEST_FILE_NAME, "w");
	fprintf(fp, "\n");
	fprintf(fp, "a\n");
	fprintf(fp, "\n");
	fprintf(fp, "b c d\n");
	fprintf(fp, "e");
	fclose(fp);

	struct file *file = new_file(TEST_FILE_NAME);
	rc = file_read(file, &tokens, &n_tokens);
	assert(rc && n_tokens == 1 && strcmp(tokens[0], "a") == 0);
	rc = file_read(file, &tokens, &n_tokens);
	assert(rc && n_tokens == 3 && strcmp(tokens[0], "b") == 0
		   && strcmp(tokens[1], "c") == 0 && strcmp(tokens[2], "d") == 0);
	rc = file_read(file, &tokens, &n_tokens);
	assert(rc && n_tokens == 1 && strcmp(tokens[0], "e") == 0);
	rc = file_read(file, &tokens, &n_tokens);
	assert(! rc);
	delete_file(file);

	unlink(TEST_FILE_NAME);
	return 0;
}
